/*
  This program demonstrates moving the camera around.
  It takes three "pictures" of a fixed scene, a front
  view, a right side view, and a top view of the scene.

  The three pictures are all stored in the same
  framebuffer. This demonstrates the idea of "viewports"
  within a framebuffer.
*/
import java.awt.Color;

public class ThreeViews
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", ThreeViews.class.getName());
         System.exit(-1);;
      }

      // Create several Model objects.
      Model m1 = new CubeModel(1, 1, 1);
      for (Triangle t : m1.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m1.modelTranslate(-4, 0, 0);

      Model m2 = new SphereModel(k, n, 1);
      for (Triangle t : m2.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m2.modelTranslate(-2, 3, 0);

      Model m3 = new ConeModel(4, 4, 1, 2);
      for (Triangle t : m3.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m3.modelTranslate(0, 0, 0);

      Model m4 = new CylinderModel(k, n, 1, 2);
      for (Triangle t : m4.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }
      m4.modelTranslate(3, 2, -2);

      Model m5 = new PanelModel(-7, 7, -7, 7, Color.white, Color.black);
      m5.doLighting = false;

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(m3);
      scene.modelList.add(m4);
      scene.modelList.add(m5);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);


      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);


      // Set a new viewport within the frambuffer.
      fb.setViewport(0, 0, 256, 256); //upper-left-hand-corner, width, height
      // Give the viewport a nice background color.
      fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
      // Set up the scene's "camera" (front view).
      double eyex=0,    eyey=2,    eyez=10;      // where the camera is
      double centerx=0, centery=2, centerz=0;    // where the camera is pointed
      double upx=0,     upy=1,     upz=0;        // where the "top" of the camera is pointed
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);
      // Render our scene into the view port within the frame buffer.
      SceneRender.renderScene(scene, fb);

      // Set a new viewport within the frambuffer.
      fb.setViewport(256, 0, 256, 256);
      // Give the viewport a nice background color.
      fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
      // Change the view (side view).
      scene.view2Identity();
      eyex=10;    eyey=2;    eyez=0;
      centerx=0; centery=2; centerz=0;
      upx=0;     upy=1;     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);
      // Render again, but into the new viewport.
      SceneRender.renderScene(scene, fb);

      // Set a new viewport within the frambuffer.
      fb.setViewport(0, 255, 256, 256);
      // Give the viewport a nice background color.
      fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
      // Change the view (top view).
      scene.view2Identity();
      eyex=0;    eyey=10;    eyez=0;
      centerx=0; centery=0; centerz=0;
      upx=0;     upy=0;     upz=-1;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);
      // Render again, but into the new viewport.
      SceneRender.renderScene(scene, fb);

      // Save the resulting image in a file.
      fb.dumpFB2File( "ThreeViews.ppm" );
   }
}